package com.miaxis.app.user.service.impl;

import java.util.*;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miaxis.app.sms.ISmsService;
import com.miaxis.app.user.domain.UserImages;
import com.miaxis.app.user.domain.UserRegister;
import com.miaxis.app.user.domain.dto.UserDto;
import com.miaxis.app.user.domain.vo.UserVo;
import com.miaxis.app.user.mapper.UserImagesMapper;
import com.miaxis.app.user.mapper.UserRegisterMapper;
import com.miaxis.app.user.service.IUserImagesService;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.constant.SmsTemplateConstants;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.domain.entity.SysUser;
import com.miaxis.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import com.miaxis.app.user.mapper.UserInfoMapper;
import com.miaxis.app.user.domain.UserInfo;
import com.miaxis.app.user.service.IUserInfoService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 用户信息Service业务层处理
 *
 * @author miaxis
 * @date 2020-12-23
 */
@Service
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements IUserInfoService {

    @Resource
    private UserInfoMapper userInfoMapper;//用户信息

    @Resource
    private UserRegisterMapper userRegisterMapper;//用户报名信息

    @Resource
    private IUserImagesService userImagesService;//用户图片信息

    @Autowired
    private ISmsService smsService;//短信服务

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 查询用户信息
     *
     * @param id 用户信息ID
     * @return 用户信息
     */
    @Override
    public UserInfo selectUserInfoById(Long id){
        return userInfoMapper.selectUserInfoById(id);
    }



    /**
     * 修改用户信息
     *
     * @param userInfo 用户信息
     * @return 结果
     */
    @Override
    public int updateUserInfo(UserInfo userInfo){
        userInfo.setUpdateTime(DateUtils.getNowDate());
        return userInfoMapper.updateUserInfo(userInfo);
    }

    /**
     * 批量删除用户信息
     *
     * @param ids 需要删除的用户信息ID
     * @return 结果
     */
    @Override
    public int deleteUserInfoByIds(Long[] ids){
        return userInfoMapper.deleteUserInfoByIds(ids);
    }


    /**
     * 修改手机号
     * @param phone
     * @param verificationCode
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Response modifyPhoneNumber(String phone,String verificationCode,SysUser user) {
        //查看验证码是否过期
        if (redisTemplate.hasKey(Constants.SMS_MODIFY_CODE_COUNT + phone) &&
                !redisTemplate.hasKey(Constants.SMS_MODIFY_CODE + phone)){
            return Response.error(500,"验证码已过期,请重新验证!");
        }
        //比对缓存中的验证码
        String smsCodeValue = redisTemplate.opsForValue().get(Constants.SMS_MODIFY_CODE + phone).toString();
        if (!verificationCode.equals(smsCodeValue)){
            return Response.error(500,"验证码错误,请重新输入");
        }

        UserInfo userInfo = new UserInfo();
        userInfo.setId(user.getUserId());
        userInfo.setMobile(phone);
        //更新绑定手机号
        userInfoMapper.updateById(userInfo);

        return Response.success();
    }

    /**
     * 保存用户报名信息
     * @param userDto
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Response saveUserInformation(UserDto userDto) {
        try {

            //查看验证码是否过期
            if (redisTemplate.hasKey(Constants.SMS_SIGNUP_CODE_COUNT + userDto.getMobile()) &&
                    !redisTemplate.hasKey(Constants.SMS_SIGNUP_CODE + userDto.getMobile())){
                return Response.error(500,"验证码已过期,请重新验证!");
            }
            //比对缓存中的验证码
            String smsCodeValue = redisTemplate.opsForValue().get(Constants.SMS_SIGNUP_CODE + userDto.getMobile()).toString();
            if (!userDto.getVerificationCode().equals(smsCodeValue)){
                return Response.error(500,"验证码错误,请重新输入");
            }

            UserInfo userInfo = new UserInfo();
            UserRegister userRegister = new UserRegister();
            //用户信息表
            userInfo.setName(userDto.getName());
            userInfo.setSex(userDto.getSex());
            userInfo.setSfzmlx(1l);
            userInfo.setIdCard(userDto.getIdCard());
            userInfo.setMobile(userDto.getMobile());
            userInfo.setInscode(userDto.getInscode());
            userInfo.setPxcx(userDto.getPxcx());
            userInfo.setSchoolClassTypeId(userDto.getSchoolClassTypeId());
            userInfo.setCoachnum(userDto.getCoachnum());
            userInfo.setStatus(0);
            userInfoMapper.insert(userInfo);


            //用户报名信息表
            userRegister.setUserId(userInfo.getId());
            userRegister.setInscode(userDto.getInscode());
            userRegister.setBmcx(userDto.getPxcx());
            userRegister.setCoachnum(userDto.getCoachnum());
            userRegister.setClassId(userDto.getSchoolClassTypeId());
            userRegisterMapper.insert(userRegister);

            //用户图片信息
            List<UserImages> userImagesList = new ArrayList<>();
            for (int i = 0; i < 3; i++) {
                UserImages userImages = new UserImages();
                userImages.setUserId(userInfo.getId());
                switch(i){
                    case 0 :
                        userImages.setFileUrl(userDto.getIdCardFrontUrl());
                        userImages.setFilePath(userDto.getIdCardFrontPath());
                        userImages.setImageType(1l);
                        userImagesList.add(userImages);
                        break;
                    case 1 :
                        userImages.setFileUrl(userDto.getIdCardReverseUrl());
                        userImages.setFilePath(userDto.getIdCardReversePath());
                        userImages.setImageType(2l);
                        userImagesList.add(userImages);
                        break;
                    case 2 :
                        userImages.setFileUrl(userDto.getFacesUrl());
                        userImages.setFilePath(userDto.getFacesPath());
                        userImages.setImageType(3l);
                        userImagesList.add(userImages);
                        break;
                    default :
                        break;
                }
            }
            userImagesService.saveBatch(userImagesList);
            return Response.success(userInfo);
        }catch (Exception e){
            throw new RuntimeException(e);
        }

    }

    /**
     * 注册报名获取短信验证码
     * @param type
     * @param phone
     * @return
     */
    @Override
    public Response sendSignUpVerificationCode(String type, String phone) {
        Map<String, String> map = new HashMap<>();
        map.put("phone",phone);
        map.put("type",type);
        map.put("keyPrefix", Constants.SMS_SIGNUP_CODE);
        map.put("verificationCount",Constants.SMS_SIGNUP_CODE_COUNT);
        map.put("smsTemplateConstants",SmsTemplateConstants.signUp);
        return smsService.sendVerificationCode(map);
    }


    /**
     * 修改手机号获取短信验证码
     * @param user
     * @param type
     * @param phone
     * @return
     */
    @Override
    public Response sendModifyVerificationCode(SysUser user, String type, String phone) {
        if (user.getPhonenumber().equals(phone)){
            return Response.error(500,"该手机号与当前绑定的手机号相同");
        }

        Map<String, String> map = new HashMap<>();
        map.put("phone",phone);
        map.put("type",type);
        map.put("keyPrefix", Constants.SMS_LOGIN_CODE);
        map.put("verificationCount",Constants.SMS_LOGIN_CODE_COUNT);
        map.put("smsTemplateConstants",SmsTemplateConstants.login);

        return smsService.sendVerificationCode(map);
    }

    /**
     * 获取我的报名信息
     * @param userId
     * @return
     */
    @Override
    public Response getMyRegistrationInformation(String userId) {
        UserVo userVo = userInfoMapper.getMyRegistrationInformation(userId);
        return Response.success(userVo);
    }


}
